/*
 * Copyright (c) 2004 by Beacon Information Technology, Inc.
 *
 * Licensor (Beacon IT) hereby grants You a world-wide, royalty-free,
 * non-exclusive, perpetual, sublicenseable license to do the
 * following:
 *
 *    1. to reproduce the Original Work in copies;
 *    2. to prepare derivative works ("Derivative Works") based
 *       upon the Original Work;
 *    3. to distribute copies of the Original Work and Derivative
 *       Works to the public; and
 *    4. to display the Original Work publicly. 
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
package portlets;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.GenericPortlet;
import javax.portlet.PortletContext;
import javax.portlet.PortletMode;
import javax.portlet.PortletPreferences;
import javax.portlet.PortletRequestDispatcher;
import javax.portlet.PortletSession;
import javax.portlet.PortletURL;
import javax.portlet.ReadOnlyException;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;
import javax.portlet.ValidatorException;
import javax.portlet.WindowState;
import javax.portlet.PortletException;
import javax.xml.transform.Templates;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import java.io.IOException;
import java.io.Writer;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Locale;
import java.util.ResourceBundle;

/**
 * BirthdaysPortletはいろいろポートレットAPIを利用するサンプル
 * ポートレットです。
 *
 * @author ウェストベイ マイケル 技師 [(株)ビーコンIT プロダクト開発部]
 * @version 1.0
 */
public class BirthdaysPortlet extends GenericPortlet {

    /** データフィルタ(トランスフォーム)の日付フォーマットを指定する。 */
    static final SimpleDateFormat dateFormat = new SimpleDateFormat("MM/dd");
    
    /** XSLテンプレート定義です。 */
    static Templates templates = null;

    /* 固定変数を初期化する。 */
    static {
        try {
            URL url = BirthdaysPortlet.class.getClassLoader().getResource("players.xsl");
            StreamSource src = new StreamSource(url.openStream());
            templates = TransformerFactory.newInstance().newTemplates(src);
        } catch (IOException ex) {
            System.out.println("Failed to read players.xsl.");
        } catch (TransformerConfigurationException ex) {
            System.out.println(ex.toString());
        }
    }

    /**
     * VIEWモードの表示するメソッドです。前の日、今日、と次の日の
     * 誕生日である選手を出力する。
     *
     * この例では、セッションでポータルのロケールを上書きしています。
     * セッションを利用するサンプルのためにこのようにインプリメント
     * しています。実際に、国際化がポータルに任せる方がいいです。
     *
     * @param request ポートレットのリクエストオブジェクト
     * @param response ポートレットのレスポンス(返答)オブジェクト
     * @throws javax.portlet.PortletException リクエストを処理できない場合
     * @throws java.io.IOException ストリームにI/Oの問題がある場合
     */
    public void doView(RenderRequest request, RenderResponse response)
            throws PortletException, IOException {

        // 返答のMIMEタイプを指定する
        response.setContentType("text/html");
        
        // 最初は、セッションからロケールを取得してみる
        PortletSession session = request.getPortletSession();
        Locale locale = (Locale)session.getAttribute("locale",
            PortletSession.PORTLET_SCOPE);

        // ロケールがセッションになければ、ポータルから取得
        if (locale == null) {
            locale = request.getLocale();
        }

        // 変更先のロケールを指定する
        Locale locale2 = Locale.JAPANESE.equals(locale)?
            Locale.ENGLISH:Locale.JAPANESE;
        
        // このポートレットのリソースバンドルを取得する
        ResourceBundle rb = getResourceBundle(locale);
        
        // ダイナミックにタイトルを指定する
        response.setTitle(rb.getString("javax.portlet.title"));

        // ポートレットのプリファレンス(好み)を取得する
        PortletPreferences prefs = request.getPreferences();
        String nameType = prefs.getValue("name-type", "full");
        String listBy = prefs.getValue("list-by", "");

        // 出力Writerを取得する
        Writer out = response.getWriter();
        
        // 前の日の日付を準備する
        Calendar cal = Calendar.getInstance();
        cal.add(Calendar.DAY_OF_YEAR, -1);
        
        // *** ここから出力する ***

        // 言語の変更するアクションリンク
        PortletURL action = response.createActionURL();
        action.setParameter("set-language", locale2.getLanguage());
        out.write("<div class=\"portlet-menu-item\" style=\"text-align: right;\">");
        out.write("<a href=\"");
        out.write(action.toString());
        out.write("\">");
        out.write(locale2.getDisplayLanguage(locale));
        out.write("</a></div>");
        
        try {
            // クライアントのロケールのデータファイルを取得する
            URL url = BirthdaysPortlet.class.getClassLoader()
                .getResource("players_"+locale.getLanguage()+".xml");

            // トランスフォームのパラメータを指定する
            Transformer trans = templates.newTransformer();
            if (nameType != null) {
                trans.setParameter("name-type", nameType);
            }
            if (listBy != null) {
                trans.setParameter("list-by", listBy);
            }
            
            // 3日間の間にループする
            for (int i=0; i<3; i++) {
                // 日付のヘッダーを出力する
                out.write("<div class=\"portlet-section-header\">");
                out.write(rb.getString("header"+i));
                out.write(" (");
                out.write(dateFormat.format(cal.getTime()));
                out.write(")");
                out.write("</div>");

                // 日付のトランスフォームパラメータを指定して、
                trans.setParameter("date", dateFormat.format(cal.getTime()));
                //   トランスフォームする
                trans.transform(new StreamSource(url.openStream()), 
                    new StreamResult(out));
                
                // 次の日
                cal.add(Calendar.DAY_OF_YEAR, 1);
            }
        } catch (Throwable th) {
            // エラーがあった場合、トランスフォームが失敗した国際化された
            //   メッセージを出力する
            out.write("<div class=\"portlet-msg-error\">");
            out.write(rb.getString("transformFailed"));
            out.write("</div>");
            th.printStackTrace();
        }
    }
    
    /**
     * EDITモードの表示するメソッドです。JSPページへディスパッチする。
     * 
     * @param request ポートレットのリクエストオブジェクト
     * @param response ポートレットのレスポンス(返答)オブジェクト
     * @throws javax.portlet.PortletException リクエストを処理できない場合
     * @throws java.io.IOException ストリームにI/Oの問題がある場合
     */
    protected void doEdit(RenderRequest request, RenderResponse response)
            throws PortletException, IOException {

        PortletContext context = getPortletContext();
        PortletRequestDispatcher rd = context.getRequestDispatcher(
            "/WEB-INF/jsp/edit.jsp");
        rd.include(request, response);
    }

    /**
     * アクションを処理する。
     *
     * このポートレットは 2つアクションをハンドルします。最初は、
     * <code>set-language</code>アクションで言語を変更します。
     *
     * 次のアクションは、ポートレットの表示プリファレンスを指定します。
     * 実際なポートレットで一般ユーザがポートレットのプリファレンスを
     * 指定することはありません。管理者しか設定することができないよう
     * にコーディングしなければいけません。例のためにこのように作りま
     * した。気を付けてください。
     *  
     * @param request アクションのリクエストオブジェクト
     * @param response アクションのレスポンス(返答)オブジェクト
     * @throws javax.portlet.PortletException リクエストを処理できない場合
     * @throws java.io.IOException ストリームにI/Oの問題がある場合
     */
    public void processAction(ActionRequest request, ActionResponse response)
            throws PortletException, java.io.IOException {

        // エラーメッセージがある場合、クリアする
        PortletSession session = request.getPortletSession();
        session.removeAttribute("error-message", PortletSession.PORTLET_SCOPE);
        
        // 「set-language」アクションを処理する
        String lang;
        if ((lang = request.getParameter("set-language")) != null) {
            session.setAttribute("locale", 
                lang.equals(Locale.JAPANESE.getLanguage())?
                Locale.JAPANESE:Locale.ENGLISH, PortletSession.PORTLET_SCOPE);
        }

        // 「apply-prefs」アクションを処理する
        else if (request.getParameter("apply-prefs") != null) try {

            // ポートレットのプリファレンスを取得して、
            PortletPreferences prefs = request.getPreferences();

            //   指定して、
            prefs.setValue("list-by", request.getParameter("group"));
            prefs.setValue("name-type", request.getParameter("name"));

            //   保存する
            prefs.store();
        } catch (ReadOnlyException ex) {
            // 保存できない例外をエラーメッセージに指定して、
            session.setAttribute("error-message", ex.getMessage());
            //   編集を継続
            response.setPortletMode(PortletMode.EDIT);
        } catch (ValidatorException ex) {
            // プリファレンスの値が無効のエラーメッセージを指定して、
            session.setAttribute("error-message", ex.getMessage());
            //   編集を継続
            response.setPortletMode(PortletMode.EDIT);
        }
    }
}
