<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet
        xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
        version="1.1">

    <xsl:output
            method="html"
            version="4.0"
            encoding="UTF-8"
            indent="yes" />

    <xsl:param name="name-type">full</xsl:param>
    <xsl:param name="list-by">name</xsl:param>
    <xsl:param name="date"/>

    <xsl:template match="/players">
        <xsl:choose>
            <xsl:when test="$list-by='birthplace'">
                <xsl:call-template name="list-by-birthplace"/>
            </xsl:when>
            <xsl:when test="$list-by='team'">
                <xsl:call-template name="list-by-team"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:call-template name="list-by-name"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <xsl:template name="list-by-birthplace">
        <xsl:for-each select="player[substring-after(property[@name='birthday'], '/')=$date]">
            <xsl:sort select="property[@name='birthplace']"/>
            <xsl:apply-templates select="."/>
        </xsl:for-each>
    </xsl:template>

    <xsl:template name="list-by-name">
        <xsl:for-each select="player[substring-after(property[@name='birthday'], '/')=$date]">
            <xsl:sort select="property[@name='surname']"/>
            <xsl:sort select="property[@name='given-name']"/>
            <xsl:apply-templates select="."/>
        </xsl:for-each>
    </xsl:template>

    <xsl:template name="list-by-team">
        <xsl:for-each select="player[substring-after(property[@name='birthday'], '/')=$date]">
            <xsl:sort select="property[@name='team']"/>
            <xsl:apply-templates select="."/>
        </xsl:for-each>
    </xsl:template>

    <xsl:template match="player">
        <xsl:element name="div">
            <xsl:attribute name="class">portlet-section-body</xsl:attribute>
            <xsl:choose>
                <xsl:when test="$list-by='birthplace'">
                    <xsl:value-of select="property[@name='birthplace']"/>
                    <xsl:text>: </xsl:text>
                </xsl:when>
                <xsl:when test="$list-by='team'">
                    <xsl:value-of select="property[@name='team']"/>
                    <xsl:text>: </xsl:text>
                </xsl:when>
            </xsl:choose>
            <xsl:element name="a">
                <xsl:attribute name="href">
                    <xsl:text>http://www.japanesebaseball.com/players/player.jsp?PlayerID=</xsl:text>
                    <xsl:value-of select="property[@name='player-id']"/>
                </xsl:attribute>
                <xsl:choose>
                    <xsl:when test="$name-type='reg'">
                        <xsl:value-of select="property[@name='reg-name']"/>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:value-of select="property[@name='surname']"/>
                        <xsl:text> </xsl:text>
                        <xsl:value-of select="property[@name='given-name']"/>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:element>
        </xsl:element>
    </xsl:template>

</xsl:stylesheet>
